/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Device
 * Copyright (c) 2004-2014 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    VirtualCOM.c
 * Purpose: USB Device virtual COM port demonstration
 *----------------------------------------------------------------------------*/

#include "cmsis_os.h"
#include "rl_usb.h"

#include "Driver_USART.h"

extern ARM_DRIVER_USART Driver_USART2;

#define BUF_SZ 128

// UART -> USB
uint8_t  uart2usb_buf[BUF_SZ];
volatile uint32_t uart2usb_in;
volatile uint32_t uart2usb_out;

// USB -> UART
uint8_t  usb2uart_buf[BUF_SZ];
volatile bool uart_send_busy;

         bool bridge_ready = false;
volatile bool uart_setup   = false;

void USART2_Callback (uint32_t event) {

  if ( event &  ARM_USART_EVENT_SEND_COMPLETE) {
    uart_send_busy = false;
  }
  if ( event &  ARM_USART_EVENT_RECEIVE_COMPLETE) {
    uart2usb_in += BUF_SZ;
    Driver_USART2.Receive(uart2usb_buf, BUF_SZ);
  }
}

int main (void) {
  volatile int32_t cnt, limit;
  int32_t  ret_val;

  USBD_Initialize    (0);               // USB Device 0 Initialization
  USBD_Connect       (0);               // USB Device 0 Connect

  while (1) {
    if (USBD_Configured (0)) {
      if (uart_setup) {
        bridge_ready   = true;
        uart_setup     = false;
        uart_send_busy = false;
        uart2usb_in    = 0;
        uart2usb_out   = 0;
        Driver_USART2.Receive(uart2usb_buf, BUF_SZ);
      }
      if (bridge_ready) {

        // UART -> USB
        cnt  = (uart2usb_in + Driver_USART2.GetRxCount()) - uart2usb_out;
        if (cnt > 0) {
          limit  = BUF_SZ - (uart2usb_out & (BUF_SZ - 1));
          if (cnt > limit) { cnt = limit; }

          ret_val = USBD_CDC_ACM_WriteData (0, (const uint8_t *) (uart2usb_buf + (uart2usb_out & (BUF_SZ - 1))), cnt);
          if (ret_val > 0) { uart2usb_out += ret_val; }
        }

        // USB -> UART
        if (uart_send_busy == false) {
          cnt = USBD_CDC_ACM_ReadData(0, usb2uart_buf, BUF_SZ);
          if (cnt > 0) {
            uart_send_busy = true;
            Driver_USART2.Send(usb2uart_buf, cnt);
          }
        }
      }
    }
    osDelay (5);
  }
}
